/**
 * \addtogroup uipopt
 * @{
 */

/**
 * \name Project-specific configuration options
 * @{
 *
 * uIP has a number of configuration options that can be overridden
 * for each project. These are kept in a project-specific uip-conf.h
 * file and all configuration names have the prefix UIP_CONF.
 */

/*
 * Copyright (c) 2006, Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack
 *
 * $Id: uip-conf.h,v 1.3 Broadcom SDK $
 */

#ifndef __UIP_CONF_H__
#define __UIP_CONF_H__

#include "system.h"

/**
 * 8 bit datatype
 *
 * This typedef defines the 8-bit type used throughout uIP.
 *
 * \hideinitializer
 */
typedef uint8 u8_t;
typedef uint8 uint8_t;
typedef char int8_t;

/**
 * 16 bit datatype
 *
 * This typedef defines the 16-bit type used throughout uIP.
 *
 * \hideinitializer
 */
typedef uint16 u16_t;
typedef uint16 uint16_t;

/**
 * 32 bit datatype
 *
 * This typedef defines the 32-bit type used throughout uIP.
 *
 * \hideinitializer
 */
typedef uint32 u32_t;
typedef uint32 uint32_t;

/**
 * Statistics datatype
 *
 * This typedef defines the dataype used for keeping statistics in
 * uIP.
 *
 * \hideinitializer
 */
typedef unsigned short uip_stats_t;

/**
 * Maximum number of TCP connections.
 *
 * \hideinitializer
 */
#define UIP_CONF_MAX_CONNECTIONS        10

/**
 * Maximum number of listening TCP ports.
 *
 * \hideinitializer
 */
#define UIP_CONF_MAX_LISTENPORTS        2

/**
 * uIP buffer size.
 *
 * \hideinitializer
 */
#define UIP_CONF_BUFFER_SIZE            1624

/**
 * CPU byte order.
 *
 * \hideinitializer
 */
#if CFG_LITTLE_ENDIAN
#define UIP_CONF_BYTE_ORDER             UIP_LITTLE_ENDIAN
#endif /* CFG_BIG_ENDIAN */

/**
 * Logging on or off
 *
 * \hideinitializer
 */
#define UIP_CONF_LOGGING                0

/**
 * UDP support on or off
 *
 * \hideinitializer
 */
#define UIP_CONF_UDP                    1

/**
 * TCP support on or off
 *
 * \hideinitializer
 */
#define UIP_CONF_TCP                    1

/**
 * UDP checksums on or off
 *
 * \hideinitializer
 */
#define UIP_CONF_UDP_CHECKSUMS          1

/**
 * Determines if support for opening connections from uIP should be
 * compiled in.
 *
 * If the applications that are running on top of uIP for this project
 * do not need to open outgoing TCP connections, this configuration
 * option can be turned off to reduce the code size of uIP.
 *
 * \hideinitializer
 */
#define UIP_CONF_ACTIVE_OPEN            0

/**
 * Broadcast support.
 *
 * This flag configures IP broadcast support. This is useful only
 * together with UDP.
 *
 * \hideinitializer
 *
 */
#define UIP_CONF_BROADCAST              1

/**
 * uIP statistics on or off
 *
 * \hideinitializer
 */
#define UIP_CONF_STATISTICS             0

/*
 * IPV6 functionality
 */
#ifndef UIP_CONF_IPV6
#define UIP_CONF_IPV6                   0
#endif /* !UIP_CONF_IPV6 */


/*
 * Special handling for ICMPv6 packets
 */
#define UIP_CONF_ICMP6                  0

/*
 * Architecture specific 32-bit operation
 */
#define UIP_ARCH_ADD32                  1
#define UIP_ARCH_CHKSUM                 1

/*
 * For neighbor management
 */
#define UIP_NEIGHBOR_CONF_ADDRTYPE      0

/*
 * L2 address type
 */
#define UIP_CONF_LL_802154              0
#define UIP_CONF_LL_80211               0 

/*
 * Things we don't need
 */
#define UIP_CONF_ROUTER                 0
#define UIP_CONF_ND6_SEND_RA            0
#define UIP_CONF_IPV6_RPL               0
#define UIP_CONF_DS6_ROUTE_NBU          0
#define UIP_UDP_SEND_UNREACH_NOPORT     0

/* Here we include the header file for the application(s) we use in
   our project. */
#include "uip_task.h"

#endif /* __UIP_CONF_H__ */

/** @} */
/** @} */
